@@ -67,6 +67,7 @@
 #define MAX_ASYNC_JOBS 10
 
 struct TopLeftTextReadState {
+	gboolean large;
 	NautilusFile *file;
 	EelReadFileHandle *handle;
 };
@@ -534,6 +535,11 @@ nautilus_directory_set_up_request (Request *request,
 		request->file_info = TRUE;
 	}
 	
+	if (file_attributes & NAUTILUS_FILE_ATTRIBUTE_LARGE_TOP_LEFT_TEXT) {
+		request->large_top_left_text = TRUE;
+		request->file_info = TRUE;
+	}
+	
 	if (file_attributes & NAUTILUS_FILE_ATTRIBUTE_ACTIVATION_URI) {
 		request->file_info = TRUE;
 		request->link_info = TRUE;
@@ -856,13 +862,12 @@ dequeue_pending_idle_callback (gpointer callback_data)
 				nautilus_file_ref (file);
 				changed_files = g_list_prepend (changed_files, file);
 			}
-			nautilus_file_ref (file);			
 		} else {
 			/* new file, create a nautilus file object and add it to the list */
 			file = nautilus_file_new_from_info (directory, file_info);
 			nautilus_directory_add_file (directory, file);			
+			added_files = g_list_prepend (added_files, file);
 		}
-		added_files = g_list_prepend (added_files, file);
 	}
 
 	/* If we are done loading, then we assume that any unconfirmed
@@ -1486,9 +1491,7 @@ nautilus_directory_get_info_for_new_files (NautilusDirectory *directory,
 	gnome_vfs_async_get_file_info
 		(&handle,
 		 vfs_uri_list,
-		 (GNOME_VFS_FILE_INFO_GET_MIME_TYPE
-		  | GNOME_VFS_FILE_INFO_FOLLOW_LINKS
-		  | GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS),
+		 NAUTILUS_FILE_DEFAULT_FILE_INFO_OPTIONS,
 		 GNOME_VFS_PRIORITY_DEFAULT,
 		 new_files_callback,
 		 directory);
@@ -1615,6 +1618,21 @@ wants_top_left (const Request *request)
 	return request->top_left_text;
 }
 
+static gboolean
+lacks_large_top_left (NautilusFile *file)
+{
+	return file->details->file_info_is_up_to_date &&
+		(!file->details->top_left_text_is_up_to_date ||
+		 file->details->got_large_top_left_text != file->details->got_top_left_text)
+		&& nautilus_file_should_get_top_left_text (file);
+}
+
+static gboolean
+wants_large_top_left (const Request *request)
+{
+	return request->large_top_left_text;
+}
+
 static gboolean
 lacks_info (NautilusFile *file)
 {
@@ -1768,6 +1786,12 @@ request_is_satisfied (NautilusDirectory *directory,
 			return FALSE;
 		}
 	}
+	
+	if (request->large_top_left_text) {
+		if (has_problem (directory, file, lacks_large_top_left)) {
+			return FALSE;
+		}
+	}
 
 	if (request->deep_count) {
 		if (has_problem (directory, file, lacks_deep_count)) {
@@ -2007,9 +2031,7 @@ start_monitoring_file_list (NautilusDirectory *directory)
 	gnome_vfs_async_load_directory
 		(&directory->details->directory_load_in_progress, /* handle */
 		 directory->details->uri,                         /* uri */
-		 (GNOME_VFS_FILE_INFO_GET_MIME_TYPE	          /* options */
-		  | GNOME_VFS_FILE_INFO_FOLLOW_LINKS
-		  | GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS),
+		 NAUTILUS_FILE_DEFAULT_FILE_INFO_OPTIONS,
 		 DIRECTORY_LOAD_ITEMS_PER_CALLBACK,               /* items_per_notification */
 		 GNOME_VFS_PRIORITY_DEFAULT,
 		 directory_load_callback,                         /* callback */
@@ -2598,11 +2620,13 @@ top_left_read_callback (GnomeVFSResult result,
 	file_details->top_left_text_is_up_to_date = TRUE;
 	g_free (file_details->top_left_text);
 	if (result == GNOME_VFS_OK) {
-		file_details->top_left_text = nautilus_extract_top_left_text (file_contents, bytes_read);
+		file_details->top_left_text = nautilus_extract_top_left_text (file_contents, directory->details->top_left_read_state->large, bytes_read);
 		file_details->got_top_left_text = TRUE;
+		file_details->got_large_top_left_text = directory->details->top_left_read_state->large;
 	} else {
 		file_details->top_left_text = NULL;
 		file_details->got_top_left_text = FALSE;
+		file_details->got_large_top_left_text = FALSE;
 	}
 
 	g_free (file_contents);
@@ -2617,11 +2641,20 @@ top_left_read_more_callback (GnomeVFSFileSize bytes_read,
 			     const char *file_contents,
 			     gpointer callback_data)
 {
-	g_assert (NAUTILUS_IS_DIRECTORY (callback_data));
+	NautilusDirectory *directory;
+
+	directory = NAUTILUS_DIRECTORY (callback_data);
+
 
 	/* Stop reading when we have enough. */
-	return bytes_read < NAUTILUS_FILE_TOP_LEFT_TEXT_MAXIMUM_BYTES
-		&& count_lines (file_contents, bytes_read) <= NAUTILUS_FILE_TOP_LEFT_TEXT_MAXIMUM_LINES;
+	if (directory->details->top_left_read_state->large) {
+		return bytes_read < NAUTILUS_FILE_LARGE_TOP_LEFT_TEXT_MAXIMUM_BYTES
+			&& count_lines (file_contents, bytes_read) <= NAUTILUS_FILE_LARGE_TOP_LEFT_TEXT_MAXIMUM_LINES;
+	} else {
+		return bytes_read < NAUTILUS_FILE_TOP_LEFT_TEXT_MAXIMUM_BYTES
+			&& count_lines (file_contents, bytes_read) <= NAUTILUS_FILE_TOP_LEFT_TEXT_MAXIMUM_LINES;
+	}
+		
 }
 
 static void
@@ -2636,7 +2669,10 @@ top_left_stop (NautilusDirectory *directory)
 			g_assert (file->details->directory == directory);
 			if (is_needy (file,
 				      lacks_top_left,
-				      wants_top_left)) {
+				      wants_top_left) ||
+			    is_needy (file,
+				      lacks_large_top_left,
+				      wants_large_top_left)) {
 				return;
 			}
 		}
@@ -2651,15 +2687,25 @@ top_left_start (NautilusDirectory *directory,
 		NautilusFile *file)
 {
 	char *uri;
+	gboolean needs_large;
 
 	if (directory->details->top_left_read_state != NULL) {
 		return;
 	}
+	
+	needs_large = FALSE;
+
+	if (is_needy (file,
+		      lacks_large_top_left,
+		      wants_large_top_left)) {
+		needs_large = TRUE;
+	}
 
 	/* Figure out which file to read the top left for. */
-	if (!is_needy (file,
-		       lacks_top_left,
-		       wants_top_left)) {
+	if (!(needs_large ||
+	      is_needy (file,
+			lacks_top_left,
+			wants_top_left))) {
 		return;
 	}
 
@@ -2667,6 +2713,7 @@ top_left_start (NautilusDirectory *directory,
 		g_free (file->details->top_left_text);
 		file->details->top_left_text = NULL;
 		file->details->got_top_left_text = FALSE;
+		file->details->got_large_top_left_text = FALSE;
 		file->details->top_left_text_is_up_to_date = TRUE;
 
 		nautilus_directory_async_state_changed (directory);
@@ -2679,6 +2726,7 @@ top_left_start (NautilusDirectory *directory,
 
 	/* Start reading. */
 	directory->details->top_left_read_state = g_new0 (TopLeftTextReadState, 1);
+	directory->details->top_left_read_state->large = needs_large;
 	directory->details->top_left_read_state->file = file;
 	uri = nautilus_file_get_uri (file);
 	directory->details->top_left_read_state->handle = eel_read_file_async
@@ -2820,9 +2868,7 @@ file_info_start (NautilusDirectory *directory,
 	fake_list.prev = NULL;
 	fake_list.next = NULL;
 
-	options = GNOME_VFS_FILE_INFO_GET_MIME_TYPE
-		| GNOME_VFS_FILE_INFO_FOLLOW_LINKS
-		| GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS;
+	options = NAUTILUS_FILE_DEFAULT_FILE_INFO_OPTIONS;
 	if (need_slow_mime) {
 		options |= GNOME_VFS_FILE_INFO_FORCE_SLOW_MIME_TYPE;
 	}
@@ -3479,6 +3525,10 @@ file_needs_low_priority_work_done (NautilusDirectory *directory,
 		return TRUE;
 	}
 
+	if (is_needy (file, lacks_large_top_left, wants_large_top_left)) {
+		return TRUE;
+	}
+
 	return FALSE;
 }
 
