@@ -32,7 +32,6 @@
 #include "fm-error-reporting.h"
 #include "fm-list-model.h"
 #include <string.h>
-#include <eel/eel-cell-renderer-pixbuf-list.h>
 #include <eel/eel-vfs-extensions.h>
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-gtk-macros.h>
@@ -49,7 +48,7 @@
 #include <gtk/gtktreeselection.h>
 #include <gtk/gtktreeview.h>
 #include <libegg/eggtreemultidnd.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnome/gnome-macros.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libnautilus-extension/nautilus-column-provider.h>
@@ -1045,26 +1044,28 @@ get_file_for_path_callback (NautilusTreeViewDragDest *dest,
 /* Handles an URL received from Mozilla */
 static void
 list_view_handle_url (NautilusTreeViewDragDest *dest, const char *encoded_url,
-		      GdkDragAction action, int x, int y, FMListView *view)
+		      const char *target_uri, GdkDragAction action, int x, int y, FMListView *view)
 {
 	fm_directory_view_handle_url_drop (FM_DIRECTORY_VIEW (view),
-					   encoded_url, action, x, y);
+					   encoded_url, target_uri, action, x, y);
 }
 
 static void
 list_view_handle_uri_list (NautilusTreeViewDragDest *dest, const char *item_uris,
+			   const char *target_uri,
 			   GdkDragAction action, int x, int y, FMListView *view)
 {
 	fm_directory_view_handle_uri_list_drop (FM_DIRECTORY_VIEW (view),
-						item_uris, action, x, y);
+						item_uris, target_uri, action, x, y);
 }
 
 static void
 list_view_handle_text (NautilusTreeViewDragDest *dest, const char *text,
+		       const char *target_uri,
 		       GdkDragAction action, int x, int y, FMListView *view)
 {
 	fm_directory_view_handle_text_drop (FM_DIRECTORY_VIEW (view),
-					    text, action, x, y);
+					    text, target_uri, action, x, y);
 }
 
 static void
